<?php

namespace Bydedi\Shipping\Ups\Service;

use Bydedi\Shipping\Relay;
use Bydedi\Shipping\RelayInterface;
use Bydedi\Shipping\Ups\Service\Exception\Exception;
use Bydedi\Shipping\Ups\Service\Locator\Json\Client as JsonClient;
use Bydedi\Shipping\Ups\Service\Locator\Json\RelayFetcher as JsonRelayFetcher;
use Bydedi\Shipping\Ups\Service\Locator\Json\RelayFormatter as JsonRelayFormatter;
use Bydedi\Shipping\Ups\Service\Locator\Locator;
use Bydedi\Shipping\Ups\Service\Locator\RequestFormatter;
use Bydedi\Shipping\Ups\Service\Locator\Xml\Client as XmlClient;
use Bydedi\Shipping\Ups\Service\Locator\Xml\RelayFormatter as XmlRelayFormatter;
use Bydedi\Shipping\Ups\Service\Locator\Xml\RelayFetcher as XmlRelayFetcher;
use GuzzleHttp\Client;
use InvalidArgumentException;

/**
 * Class Ups
 *
 * @package Bydedi\Shipping\Ups\Service
 */
class Ups implements RelayInterface
{

    /**
     * @var Locator
     */
    protected $locator;


    /**
     * Ups constructor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        $upsConfig = $config['ups'];
        $upsConfig += [
            'format' => 'json',
            'env' => getenv('APP_ENV')
        ];

        $env = $upsConfig['env'];

        switch ($upsConfig['format']) {
            case 'json':
                $relayFetcher = new JsonRelayFetcher(new JsonClient($env, new Client()), new JsonRelayFormatter());
                break;

            case 'xml':
                $relayFetcher = new XmlRelayFetcher(new XmlClient($env, new Client()), new XmlRelayFormatter());
                break;

            default:
                $relayFetcher = new JsonRelayFetcher(new JsonClient($env, new Client()), new JsonRelayFormatter());
                $error_msg    = "Invalid format given '{$upsConfig['format']}', the list of format is: json, xml";
                if ($env !== 'prod') {
                    throw new InvalidArgumentException($error_msg);
                } else {
                    trigger_error($error_msg, E_USER_WARNING);
                }
                break;
        }

        $this->locator = new Locator(
            new RequestFormatter($upsConfig['api_key'], $upsConfig['identification'], $upsConfig['password']),
            $relayFetcher
        );
    }


    /**
     * @param  array $formvars
     * @return Relay[]
     */
    public function getList($formvars = [])
    {
        $formvars += [
            'rue'         => '',
            'ville'       => '',
            'code_postal' => '',
            'pays'        => '',
            'telephone'   => '',
            'locale'      => 'fr_FR',
        ];

        try {
            return $this->locator->getRelays(
                $formvars['rue'],
                $formvars['ville'],
                $formvars['code_postal'],
                $formvars['pays'],
                $formvars['telephone'],
                $formvars['locale']
            );
        } catch (Exception $exception) {
            return [];
        }
    }


    /**
     * @param string $relayId
     * @param string $countryIso
     *
     * @return Relay
     */
    public function getRelay($relayId, $countryIso = '')
    {
        return $this->locator->getRelay($relayId, '', $countryIso);
    }


    /**
     * @param  array $relayData
     * @return Relay
     */
    public function formatRelay(array $relayData)
    {
        $jsonFormatter = new JsonRelayFormatter();

        return $jsonFormatter->makeRelay(json_encode(json_decode($relayData)));
    }
}
